import express, { Express } from 'express';
import mongoose from 'mongoose';
import cors from 'cors';
import helmet from 'helmet';
import ApiRoutes from './src/router/router';
import { globalErrorHandler } from './src/middlewares/GlobalErrorHandler';
import { decodeJwt } from './src/middlewares/AuthMiddleware';
import bcrypt from 'bcryptjs';
import { User } from './src/models/UserModel';
import 'dotenv/config'
import ejs from 'ejs'
import path from 'path';

async function startApp() {
    try {
        checkEnvironmentVariables();
        await establishDatabaseConnection();
        await createSuperAdmin();

        const app: Express = express();

        app.set('view engine', 'ejs')
        app.set('views', path.join(__dirname, 'src', 'views'))
        app.use(cors());
        app.use(helmet());
        app.use(express.json());
        app.use(express.urlencoded({ extended: true }));
        app.use(decodeJwt);

        app.use('/api', ApiRoutes);

        app.use(globalErrorHandler);

        app.listen(process.env.PORT, () => {
            console.log('🚀 Server is running on port:', process.env.PORT);
        });
    } catch (err) {
        console.log('❌ Error starting app ❌');
        console.log(err);
        process.exit(1);
    }
}

startApp();

process.on('SIGTERM', async () => {
    console.log('Closing MongoDB connection');
    await mongoose.disconnect();
    console.log('MongoDB connection closed');
    process.exit(0);
});

function checkEnvironmentVariables() {
    const requiredEnvVariables = [
        'DATABASE_URL',
        'NODE_ENV',
        'PORT',
        'JWT_SECRET',
        'SUPER_ADMIN_EMAIL',
        'SUPER_ADMIN_PASSWORD',
        'SUPER_ADMIN_VAPI_KEY',
        'SUPER_ADMIN_PHONE_NUMBER_ID',
        'SMTP_HOST',
        'SMTP_USERNAME',
        'SENDGRID_API_KEY',
        'EMAIL_FROM',
        'WEBHOOK_KEY',
        'SEND_EMAILS'
    ];

    requiredEnvVariables.forEach((envVar) => {
        let missingEnvVariables = requiredEnvVariables.filter((envVar) => !process.env[envVar]);
        if (missingEnvVariables.length) {
            throw new Error(`❌ Missing environment variables: ${missingEnvVariables.join(', ')}`);
        }
    });
}

async function establishDatabaseConnection(retries: number = 5, delay: number = 1000) {
    const databaseUrl = process.env.DATABASE_URL as string;

    while (retries) {
        try {
            await mongoose.connect(databaseUrl);
            console.log('✅ Database connection established');
            return;
        } catch (error) {
            console.error('❌ Error connecting to the database!');
            if (retries) {
                console.log(`Retrying in ${delay}ms...`);
                retries--;
                await new Promise((resolve) => setTimeout(resolve, delay));
            } else {
                throw new Error('❌ Maximum retries reached! Database Connection Failed. Exiting...');
            }
        }
    }
}

// Function to ensure a Super Admin exists
async function createSuperAdmin() {
    try {
        const existingSuperAdmin = await User.findOne({ role: 'SUPER_ADMIN' });

        if (!existingSuperAdmin) {
            const hashedPassword = await bcrypt.hash(process.env.SUPER_ADMIN_PASSWORD as string, 10);

            // Create new Super Admin user
            const superAdmin = new User({
                name: 'Super Admin',
                email: process.env.SUPER_ADMIN_EMAIL,
                phoneNumberId: process.env.SUPER_ADMIN_PHONE_NUMBER_ID,
                role: 'SUPER_ADMIN',
                password: hashedPassword,
                isActive: true,
            });

            await superAdmin.save();
            console.log('✅ Super Admin user created');
        } else {
            console.log('✅ Super Admin user already exists');
        }
    } catch (err) {
        console.error('❌ Error creating Super Admin:', err);
    }
}
