import { Request, Response, NextFunction } from 'express';
import jwt from 'jsonwebtoken';
import { JwtPayload } from 'jsonwebtoken';

declare global {
    namespace Express {
        interface Request {
            jwtPayload?: AppJwtPayload;
        }
    }
}
export interface AppJwtPayload extends JwtPayload {
    userId: string;
    phoneNumberId: string,
    email: string,
    role: string;
    // Any other custom properties
}

export function decodeJwt(req: Request, res: Response, next: NextFunction) {
    const token = req.headers.authorization;
    try {
        if (token) {
            const decoded = jwt.verify(token, process.env.JWT_SECRET!) as AppJwtPayload;
            req.jwtPayload = decoded;
        }
        next();
    } catch (err) {
        res.status(401).json({ message: 'Unauthorized' });
        return;
    }
}

export function requireAuth(req: Request, res: Response, next: NextFunction) {
    if (!req.jwtPayload) {
        res.status(401).json({ message: 'Unauthorized' });
        return;
    }
    next();
}

export function restrictTo(roles: string[]) {
    return (req: Request, res: Response, next: NextFunction) => {
        if (!req.jwtPayload) {
            res.status(401).json({ message: 'Unauthorized' });
            return;
        }

        if (!roles.includes(req.jwtPayload.role)) {
            res.status(403).json({ message: 'Forbidden' });
            return;
        }

        next();
    };
}

export function vapiWebhookAuth(req: Request, res: Response, next: NextFunction) {
    try {
        if (!req.headers['x-vapi-secret'] || req.headers['x-vapi-secret'] !== process.env.WEBHOOK_KEY) {
            res.status(401).json({ message: 'Unauthorized' });
            return;
        }
        next();
    } catch (err) {
        res.status(401).json({ message: 'Unauthorized' });
        return;
    }
}