import mongoose, { Schema, Document } from 'mongoose';

// Main interface for the Call document
export interface Call extends Document {
  callId: string;  // vapi call id
  orgId: string;
  createdAt: Date;
  updatedAt?: Date;
  type?: 'inboundPhoneCall' | 'outboundPhoneCall' | 'webCall';
  costs?: any[];
  messages?: any[];
  phoneCallProvider?: string;
  phoneCallTransport?: 'sip' | 'pstn';
  status?: 'scheduled' | 'queued' | 'ringing' | 'in-progress' | 'forwarding' | 'ended';
  endedReason?: string;
  destination?: Record<string, any>;
  startedAt?: Date;
  endedAt?: Date;
  cost?: number;
  costBreakdown?: Record<string, any>;
  artifactPlan?: Record<string, any>;
  analysis?: Record<string, any>;
  monitor?: {
    listenUrl?: string;
    controlUrl?: string;
    [key: string]: any;
  };
  artifact?: Record<string, any>;
  transport?: {
    provider?: string;
    assistantVideoEnabled?: boolean;
    [key: string]: any;
  };
  phoneCallProviderId?: string;
  assistantId?: string;
  assistant?: Record<string, any>;
  assistantOverrides?: Record<string, any>;
  squadId?: string;
  squad?: Record<string, any>;
  phoneNumberId?: string;
  phoneNumber?: Record<string, any>;
  customerNumberId?: string;
  customerNumber?: Record<string, any>;
  customer?: Record<string, any>;
  name?: string;
  schedulePlan?: {
    earliestAt: Date;
    latestAt?: Date;
    [key: string]: any;
  };
  [key: string]: any; // Allow for additional fields
}

// Create the schema
const CallSchema: Schema = new Schema(
  {
    callId: { type: String, required: true, unique: true },
    orgId: { type: String, required: true },
    type: { 
      type: String, 
      enum: ['inboundPhoneCall', 'outboundPhoneCall', 'webCall'],
      required: false 
    },
    costs: [Schema.Types.Mixed], // Flexible array for cost objects
    messages: [Schema.Types.Mixed], // Flexible array for message objects
    phoneCallProvider: { type: String, required: false },
    phoneCallTransport: { 
      type: String, 
      enum: ['sip', 'pstn'],
      required: false 
    },
    status: { 
      type: String, 
      enum: ['scheduled', 'queued', 'ringing', 'in-progress', 'forwarding', 'ended'],
      required: false 
    },
    endedReason: { type: String, required: false }, // Store as string despite being enum
    destination: { type: Schema.Types.Mixed, required: false }, // Flexible object
    startedAt: { type: Date, required: false },
    endedAt: { type: Date, required: false },
    cost: { type: Number, required: false },
    costBreakdown: { type: Schema.Types.Mixed, required: false }, // Flexible object
    artifactPlan: { type: Schema.Types.Mixed, required: false }, // Flexible object
    analysis: { type: Schema.Types.Mixed, required: false }, // Flexible object
    monitor: {
      listenUrl: { type: String, required: false },
      controlUrl: { type: String, required: false },
      _id: false // Don't create an _id for this subdocument
    },
    artifact: { type: Schema.Types.Mixed, required: false }, // Flexible object
    transport: {
      provider: { type: String, required: false },
      assistantVideoEnabled: { type: Boolean, required: false },
      _id: false // Don't create an _id for this subdocument
    },
    phoneCallProviderId: { type: String, required: false },
    assistantId: { type: String, required: false },
    assistant: { type: Schema.Types.Mixed, required: false }, // Flexible object
    assistantOverrides: { type: Schema.Types.Mixed, required: false }, // Flexible object
    squadId: { type: String, required: false },
    squad: { type: Schema.Types.Mixed, required: false }, // Flexible object
    phoneNumberId: { type: String, required: false },
    phoneNumber: { type: Schema.Types.Mixed, required: false }, // Flexible object
    customerNumberId: { type: String, required: false },
    name: { 
      type: String, 
      required: false,
      maxlength: 40 // As specified in requirements
    },
    schedulePlan: {
      earliestAt: { type: Date, required: false },
      latestAt: { type: Date, required: false },
      _id: false // Don't create an _id for this subdocument
    }
  },
  {
    timestamps: false, // Automatically manage createdAt and updatedAt
    strict: false, // Allow additional fields not defined in schema
    id: true, // Enable id getter
    // toJSON: {
    //   virtuals: true, // Include virtuals when converting to JSON
    //   getters: true // Include getters when converting to JSON
    // },
    // toObject: {
    //   virtuals: true, // Include virtuals when converting to object
    //   getters: true // Include getters when converting to object
    // }
  }
);

// Create and export the model
export const Call = mongoose.model<Call>('Call', CallSchema);