import { Call } from '../models/CallModel';
import { AppError } from '../utils/AppError';
import mongoose from 'mongoose';


export class CallRepository {
    async getCallWithCallId(callId: string): Promise<Call | null> {
        try {
            let call = await Call.findOne({
                callId: callId
            })
            return call;
        } catch (err) {
            throw new AppError(500, 'ServerError', 'Something went wrong', err);
        }
    }

    async getCalls(phoneNumberId: string, limit: number, offset: number): Promise<Call[]> {
        try {
            let calls = [] as Call[];
            if (!phoneNumberId) {
                calls = await Call.find({}).sort({ startedAt: -1 }).skip(offset).limit(limit) ;
            } else {
                calls = await Call.find({ phoneNumberId }).sort({ startedAt: -1 }).skip(offset).limit(limit) ;
            }
            return calls;
        } catch (err) {
            throw new AppError(500, 'ServerError', 'Something went wrong', err);
        }
    }

    async saveCallDetails(callDetails: Call): Promise<Call> {
        try {            
            let call = await Call.create(callDetails);
            return call;
        } catch (err: any) {
            if (err instanceof mongoose.mongo.MongoServerError) {
                if (err.code === 11000) {
                    throw new AppError(400, 'ClientError', 'Call already exists');
                }
            }
            throw new AppError(500, 'ServerError', 'Something went wrong', err);
        }
    }


    async updateCallDetails(callId: string, updateData: Partial<Call>): Promise<Call> {
        try {
            const updatedCall = await Call.findOneAndUpdate(
                { callId },
                { $set: updateData },
                { new: true } // Returns the updated document
            );
            
            if (!updatedCall) {
                throw new AppError(404, 'ClientError', 'Call not found');
            }
            
            return updatedCall;
        } catch (err: any) {
            if (err instanceof mongoose.mongo.MongoServerError) {
                throw new AppError(500, 'ServerError', 'Database error', err);
            }
            throw new AppError(500, 'ServerError', 'Something went wrong', err);
        }
    }
    // async getUserWithEmail(email: string): Promise<z.infer<typeof userSchema> | null> {
    //     try {
    //         let user = await prisma.user.findUnique({
    //             where: {
    //                 email: email
    //             }
    //         });
    //         return user;
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async getUsers(limit: number, offset: number): Promise<z.infer<typeof userSchema>[]> {
    //     try {
    //         const users = await prisma.user.findMany({
    //             take: limit,
    //             skip: offset
    //         });
    //         return users;
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async getUsersWithTotalCount(
    //     limit: number,
    //     offset: number
    // ): Promise<{ data: z.infer<typeof userSchema>[]; totalCount: number }> {
    //     try {
    //         const [users, totalCount] = await Promise.all([this.getUsers(limit, offset), prisma.user.count()]);

    //         return { data: users, totalCount };
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async createUser(data: z.infer<typeof userCreateSchema>) {
    //     try {
    //         const user = await prisma.user.create({
    //             data
    //         });
    //         return user;
    //     } catch (err) {
    //         if (err instanceof Prisma.PrismaClientKnownRequestError) {
    //             if (err.code === 'P2002') {
    //                 throw new AppError(400, 'ClientError', 'User already exists');
    //             }
    //         }
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async updateUser(userId: string, updatedUserData: Partial<z.infer<typeof userSchema>>) {
    //     try {
    //         const user = await prisma.user.update({
    //             where: {
    //                 id: userId
    //             },
    //             data: updatedUserData
    //         });
    //         return user;
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async deleteUser(userId: 'all' | string) {
    //     try {
    //         if (userId === 'all') {
    //             await prisma.user.deleteMany({});
    //         } else {
    //             await prisma.user.delete({
    //                 where: {
    //                     id: userId
    //                 }
    //             });
    //         }
    //         return;
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }
}

export default CallRepository;
