// import { CallRepository } from '../repository/CallRepository';
// import { userCreateSchema, userDtoSchema, userSchema } from '../schemas/UserSchema';
import { AppError } from '../utils/AppError';
import bcrypt from 'bcryptjs';
import jwt from 'jsonwebtoken';
import { AppJwtPayload } from '../middlewares/AuthMiddleware';
import axios from 'axios';
import { convertToSnakeCase } from '../utils/Utils';
import { Call } from '../models/CallModel';
import CallRepository from '../repository/CallRepository';
import CommunicationService from './CommunicationService';
import fs from 'fs';
import path from 'path';
import ejs from 'ejs'


export class CallService {
    callRepository: CallRepository;
    communicationService: CommunicationService

    constructor() {
        this.callRepository = new CallRepository();
        this.communicationService = new CommunicationService();
    }

    async getCallWithCallId(loggedInUserJwtPayload: AppJwtPayload, callId: string): Promise<Call | null> {
        try {
            const call = await this.callRepository.getCallWithCallId(callId);
            if (!call) throw new AppError(404, 'ClientError', 'Call not found');

            if (
                loggedInUserJwtPayload.role !== 'SUPER_ADMIN' &&
                call.phoneNumberId !== loggedInUserJwtPayload.phoneNumberId
            ) {
                throw new AppError(401, 'ClientError', 'You are not authorized to view this.')
            }

            // console.log("Call from service -", call)
            return call
        } catch (err) {
            console.log("Error -", err)
            throw new AppError(500, 'ServerError', 'Failed to fetch calls', err);
        }
    }

    async getCalls(loggedInUserJwtPayload: AppJwtPayload, limit: number, offset: number): Promise<Call[]> {
        try {
            // const phoneNumberId = '967fbf97-c711-44fe-864b-514659e50472'  // get from user account, for now hardcoded
            
            let calls = [] as Call[];
            if (loggedInUserJwtPayload.role == 'SUPER_ADMIN') {
                calls = await this.callRepository.getCalls('', limit, offset);
            }
            else {
                const phoneNumberId = loggedInUserJwtPayload.phoneNumberId
                calls = await this.callRepository.getCalls(phoneNumberId, limit, offset);
            }

            return calls;
        } catch (err) {
            console.log("Error -", err)
            throw new AppError(500, 'ServerError', 'Failed to fetch calls', err);
        }
    }

    async callWebhookHandler(eventObj: any) {
        try {

            if (eventObj.message.type === 'status-update') {
                const {message} = eventObj;
                const callObject: Call = {} as Call;

                callObject['callId'] = message.call.id;
                callObject['orgId'] = message.call.orgId;
                callObject['type'] = message.call.type;
                callObject['status'] = message.status
                callObject['startedAt'] = message.call.createdAt
                callObject['monitor'] = message.call.monitor
                callObject['assistantId'] = message.call.assistantId
                callObject['assistant'] = message.assistant
                
                
                if (['inboundPhoneCall', 'outboundPhoneCall'].includes(message.call.type)) {
                    callObject['phoneCallProvider'] = message.call.phoneCallProvider
                    callObject['phoneCallProviderId'] = message.call.phoneCallProviderId
                    callObject['phoneNumber'] = message.phoneNumber
                    callObject['phoneCallTransport'] = message.call.phoneCallTransport
                    callObject['phoneNumberId'] = message.call.phoneNumberId
                    callObject['customer'] = { number: message.customer.number }

                }

                // Save the call details to the database
                await this.callRepository.saveCallDetails(callObject);
            } 
            else if (eventObj.message.type === 'end-of-call-report') {
                const {message} = eventObj;
                // const jsonData = JSON.stringify(message, null, 2);
                // const filePath = path.join(__dirname, `${message.call.id}_event.json`);

                // fs.writeFile(filePath, jsonData, (err) => {
                //     if (err) {
                //         console.error('Error writing file', err);
                //     } else {
                //         console.log('JSON data has been written to the file successfully');
                //     }
                // });

                // console.log("Message -", JSON.stringify(message, null, 2))
                const callObject: Call = {} as Call;


                callObject['callId'] = message.call.id;
                callObject['summary'] = message.summary
                callObject['artifact'] = message.artifact
                callObject['startedAt'] = message.startedAt
                callObject['endedAt'] = message.endedAt
                callObject['endedReason'] = message.endedReason
                callObject['cost'] = message.cost
                callObject['costBreakdown'] = message.costBreakdown
                callObject['costs'] = message.costs
                callObject['durationMs'] = message.durationMs
                callObject['durationSeconds'] = message.durationSeconds
                callObject['durationMinutes'] = message.durationMinutes
                callObject['messages'] = message.messages
                callObject['recordingUrl'] = message.recordingUrl
                callObject['stereoRecordingUrl'] = message.stereoRecordingUrl

                callObject['orgId'] = message.call.orgId;
                callObject['type'] = message.call.type;
                callObject['status'] = 'ended';
                callObject['analysis'] = message.analysis
                callObject['monitor'] = message.monitor
                
                
                if (['inboundPhoneCall', 'outboundPhoneCall'].includes(message.call.type)) {
                    callObject['phoneCallProvider'] = message.call.phoneCallProvider
                    callObject['phoneCallProviderId'] = message.call.phoneCallProviderId
                    callObject['phoneNumber'] = message.phoneNumber
                    callObject['phoneCallTransport'] = message.call.phoneCallTransport
                    callObject['phoneNumberId'] = message.call.phoneNumberId
                    callObject['assistantId'] = message.call.assistantId
                    callObject['customer'] = { number: message.customer?.number };
                }

                // Save the call details to the database
                await this.callRepository.updateCallDetails(message.call.id, callObject);

                if (process.env.SEND_EMAILS === 'true') {
                    const templatePath = path.join(__dirname, '..', 'views', 'call-report-email-template.ejs');
                    const template = fs.readFileSync(templatePath, 'utf-8');
                    // Render the template with data
                    const htmlContent = ejs.render(template, {
                        callData: callObject,
                        currentDate: new Date().toLocaleDateString(),
                        currentYear: new Date().getFullYear()
                    });
    
                    console.log("Sending emails")
                    this.communicationService.sendEmail('Call Report', 'ujjwalsaxena774@gmail.com', htmlContent)
                }
            } else {
                throw new AppError(400, 'ClientError', 'Invalid event type');
            }

        } catch (err) {
            console.log("Error -", err)
            throw new AppError(500, 'ServerError', 'Failed to fetch calls', err);
        }
    }

    async syncWithVapi(loggedInUserJwtPayload: AppJwtPayload) {
        const response = await axios.get('https://api.vapi.ai/call', {
            headers: {
                Authorization: `Bearer ${process.env.SUPER_ADMIN_VAPI_KEY}`
            },
            params: {
                limit: 1000,
                phoneNumberId: loggedInUserJwtPayload?.phoneNumberId
            }
        })

        response.data.forEach((item: any) => {
          item.callId = item.id
          delete item.id
        })


        // delete all calls with same phoneNumberId
        await Call.deleteMany({phoneNumberId: loggedInUserJwtPayload.phoneNumberId})

        // now sync
        const result = await Call.insertMany(response.data);
        return {itemsLength: response.data.length}
    }

    // async getCalls(userId: string, limit: number, offset: number, direction: 'next' | 'prev' = 'next'): Promise<any[]> {
    //     try {
    //         const user = await this.callRepository.getUserWithId(userId);
    //         if (!user) throw new AppError(404, 'ClientError', 'User not found');

    //         // const response = await axios.get(`https://api.vapi.ai/call`, {
    //         //     headers: {
    //         //         Authorization: `Bearer ${user.vapi_key}`,
    //         //     },
    //         // });

    //         // return response.data.calls.map((call: any) => ({
    //         //     call_id: call.id,
    //         //     call_started_at: call.startedAt,
    //         //     call_type: call.type,
    //         //     call_ended_at: call.endedAt,
    //         //     recording_url: call.recordingUrl,
    //         //     customer_phone: call.customer.number,
    //         //     call_status: call.status,
    //         //     call_end_reason: call.endedReason,
    //         // }));

    //         await this.prisma.phoneCall.findMany({
    //             where: {
    //                 user_id: userId,
    //             },
    //             orderBy: {                  

    //         const calls = await this.callRepository.getCalls(userId, limit, offset, direction, timestamp);
    //         return calls.data;
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Failed to fetch calls', err);
    //     }
    // }

    // async getCallWithId(callId: string, userId: number): Promise<any | null> {
    //     try {
    //         const user = await this.callRepository.getUserWithId(userId);
    //         if (!user) throw new AppError(404, 'ClientError', 'User not found');

    //         // console.log("User vapi key", user.vapi_key)

    //         // const response = await axios.get(`https://api.vapi.ai/call/${callId}`, {
    //         //     headers: {
    //         //         Authorization: `Bearer ${user.vapi_key}`,
    //         //     },
    //         // });

    //         // const data = {
    //         //     call_id: response.data.id,
    //         //     call_started_at: response.data.startedAt,
    //         //     call_type: response.data.type,
    //         //     call_ended_at: response.data.endedAt,
    //         //     recording_url: response.data.recordingUrl,
    //         //     customer_phone: response?.data?.customer?.number,
    //         //     call_status: response.data.status,
    //         //     call_end_reason: response.data.endedReason,
    //         //     call_summary: response.data?.summary,
    //         //     messages: response.data.messages ? response.data.messages.slice(1) : []
    //         // }
    //         // return data;
    //         const phoneCall = await this.prisma.phoneCall.findUnique({
    //             where: {
    //                 id: callId,
    //                 user_id: userId, // Ensure the call belongs to the user
    //             },
    //             include: {
    //                 messages: {
    //                     orderBy: {
    //                         seconds_from_start: 'asc',
    //                     },
    //                 },
    //             },
    //         });
    
    //         if (!phoneCall) {
    //             throw new AppError(404, 'ClientError', 'Call not found or not authorized to access');
    //         }
    
    //         return phoneCall;
            
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Failed to fetch call details', err);
    //     }
    // }

    // async getUserWithId(userId: number): Promise<z.infer<typeof userDtoSchema> | null> {
    //     try {
    //         let user = await this.userRepository.getUserWithId(userId);
    //         if (user) return this.getUserDto(user);
    //         return null;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async getUsers(limit: number, offset: number, withCount: boolean): Promise<UsersResponse> {
    //     try {
    //         let users = {} as UsersResponse;

    //         if (withCount) {
    //             const response = await this.userRepository.getUsersWithTotalCount(limit, offset);
    //             users = {
    //                 type: 'withCount',
    //                 data: response.data.map((user: z.infer<typeof userSchema>) => this.getUserDto(user)),
    //                 totalCount: response.totalCount
    //             };
    //         } else {
    //             const response = await this.userRepository.getUsers(limit, offset);
    //             users = {
    //                 type: 'withoutCount',
    //                 data: response.map((user) => this.getUserDto(user))
    //             };
    //         }

    //         return users;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong while fetching users', err);
    //     }
    // }

    // async createUser(
    //     userObject: z.infer<typeof userCreateSchema>,
    //     loggedInUserJwtPayload: AppJwtPayload
    // ): Promise<z.infer<typeof userDtoSchema>> {
    //     try {
    //         // if (userObject.role === 'SUPER_ADMIN' && loggedInUserJwtPayload.role !== 'SUPER_ADMIN') {
    //         // if (loggedInUserJwtPayload.role !== 'SUPER_ADMIN') {
    //         //     throw new AppError(403, 'ClientError', 'Only superadmins are allowed to create user');
    //         // }
    //         userObject.password = await bcrypt.hash(userObject.password, 10);
    //         const user = await this.userRepository.createUser(userObject);
    //         return this.getUserDto(user);
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async updateUser(
    //     userId: number,
    //     updatedUserFields: Partial<z.infer<typeof userSchema>>,
    //     loggedInUserJwtPayload: AppJwtPayload
    // ): Promise<z.infer<typeof userDtoSchema>> {
    //     try {
    //         // if (Object.keys(updatedUserFields).includes('password'))
    //         //     throw new AppError(400, 'ClientError', 'Password updates are not allowed from here');

    //         if (updatedUserFields.role === 'SUPER_ADMIN' && loggedInUserJwtPayload.role !== 'SUPER_ADMIN') {
    //             throw new AppError(403, 'ClientError', 'Only superadmins can promote a user to SUPER_ADMIN');
    //         }
    //         if (updatedUserFields.password) {
    //             updatedUserFields.password = await bcrypt.hash(updatedUserFields.password, 10);
    //         }
    //         const user = await this.userRepository.updateUser(userId, updatedUserFields);
    //         return user;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async deleteUser(userId: 'all' | number) {
    //     try {
    //         await this.userRepository.deleteUser(userId);
    //         return true;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // getUserDto(user: z.infer<typeof userSchema>): z.infer<typeof userDtoSchema> {
    //     const { password, vapi_key, ...userDto } = user;
    //     return userDto;
    // }

    // async login(email: string, password: string): Promise<{ email: string; role: string; token: string } | null> {
    //     try {
    //         const user = await this.userRepository.getUserWithEmail(email);
    //         if (!user || !user.is_active) return null;
    //         const isPasswordCorrect = await bcrypt.compare(password, user.password);
    //         if (!isPasswordCorrect) return null;

    //         const jwtPayload = {
    //             userId: user.id,
    //             email: user.email,
    //             role: user.role
    //         };
    //         const token = jwt.sign(jwtPayload, process.env.JWT_SECRET!);
    //         return { email: user.email, role: user.role, token };
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }
}

export default CallService;