// import { CallRepository } from '../repository/CallRepository';
// import { userCreateSchema, userDtoSchema, userSchema } from '../schemas/UserSchema';
import { AppError } from '../utils/AppError';
import bcrypt from 'bcryptjs';
import jwt from 'jsonwebtoken';
import { AppJwtPayload } from '../middlewares/AuthMiddleware';
import axios from 'axios';
import { convertToSnakeCase } from '../utils/Utils';
import { User } from '../models/UserModel';
import UserRepository from '../repository/UserRepository';


export class UserService {
    userRepository: UserRepository;

    constructor() {
        this.userRepository = new UserRepository();
    }

    async getUserWithPhoneNumberId(phoneNumberId: string, loggedInUserJwtPayload: AppJwtPayload): Promise<typeof User | null> {
        try {
            if (loggedInUserJwtPayload.role !== 'SUPER_ADMIN' && phoneNumberId !== loggedInUserJwtPayload.phoneNumberId) {
                throw new AppError(403, 'ClientError', 'You are not authorized to access this resource');
            }

            const user = await this.userRepository.getUserWithPhoneNumberId(phoneNumberId);

            if (!user) throw new AppError(404, 'ClientError', 'Call not found');

            return user
        } catch (err) {
            console.log("Error -", err)
            throw new AppError(500, 'ServerError', 'Failed to fetch user', err);
        }
    }

    async getUserWithEmail(email: string, loggedInUserJwtPayload: AppJwtPayload): Promise<typeof User | null> {
        try {
            if (loggedInUserJwtPayload.role !== 'SUPER_ADMIN' && email !== loggedInUserJwtPayload.email) {
                throw new AppError(403, 'ClientError', 'You are not authorized to access this resource');
            }

            const user = await this.userRepository.getUserWithEmail(email);

            if (!user) throw new AppError(404, 'ClientError', 'Call not found');

            return user
        } catch (err) {
            console.log("Error -", err)
            throw new AppError(500, 'ServerError', 'Failed to fetch user', err);
        }
    }

    async login(email: string, password: string): Promise<{ email: string; role: string; token: string, phoneNumberId: string } | null> {
        try {
            const user = await this.userRepository.getUserWithEmail(email);
            if (!user || !user.isActive) return null;
            const isPasswordCorrect = await bcrypt.compare(password, user.password);
            if (!isPasswordCorrect) return null;

            const jwtPayload = {
                userId: user.id,
                email: user.email,
                role: user.role,
                phoneNumberId: user.phoneNumberId,
            };
            const token = jwt.sign(jwtPayload, process.env.JWT_SECRET!);
            return { email: user.email, role: user.role, token, phoneNumberId: user.phoneNumberId };
        } catch (err) {
            if (err instanceof AppError) throw err;
            throw new AppError(500, 'ServerError', 'Something went wrong', err);
        }
    }

    async createUser(name: string, email: string, phoneNumberId: string, role: 'SUPER_ADMIN' | 'USER', isActive: boolean, password: string): Promise<typeof User> {
        try {
            const userObject = {
                name,
                email,
                phoneNumberId,
                role,
                isActive,
                password,
            };
            const user = await this.userRepository.createUser(userObject);
            delete user.password
            return user;
        } catch (err) {
            if (err instanceof AppError) throw err;
            throw new AppError(500, 'ServerError', 'Something went wrong', err);
        }
    }

    // async getCalls(userId: string, limit: number, offset: number, direction: 'next' | 'prev' = 'next'): Promise<any[]> {
    //     try {
    //         const user = await this.callRepository.getUserWithId(userId);
    //         if (!user) throw new AppError(404, 'ClientError', 'User not found');

    //         // const response = await axios.get(`https://api.vapi.ai/call`, {
    //         //     headers: {
    //         //         Authorization: `Bearer ${user.vapi_key}`,
    //         //     },
    //         // });

    //         // return response.data.calls.map((call: any) => ({
    //         //     call_id: call.id,
    //         //     call_started_at: call.startedAt,
    //         //     call_type: call.type,
    //         //     call_ended_at: call.endedAt,
    //         //     recording_url: call.recordingUrl,
    //         //     customer_phone: call.customer.number,
    //         //     call_status: call.status,
    //         //     call_end_reason: call.endedReason,
    //         // }));

    //         await this.prisma.phoneCall.findMany({
    //             where: {
    //                 user_id: userId,
    //             },
    //             orderBy: {                  

    //         const calls = await this.callRepository.getCalls(userId, limit, offset, direction, timestamp);
    //         return calls.data;
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Failed to fetch calls', err);
    //     }
    // }

    // async getCallWithId(callId: string, userId: number): Promise<any | null> {
    //     try {
    //         const user = await this.callRepository.getUserWithId(userId);
    //         if (!user) throw new AppError(404, 'ClientError', 'User not found');

    //         // console.log("User vapi key", user.vapi_key)

    //         // const response = await axios.get(`https://api.vapi.ai/call/${callId}`, {
    //         //     headers: {
    //         //         Authorization: `Bearer ${user.vapi_key}`,
    //         //     },
    //         // });

    //         // const data = {
    //         //     call_id: response.data.id,
    //         //     call_started_at: response.data.startedAt,
    //         //     call_type: response.data.type,
    //         //     call_ended_at: response.data.endedAt,
    //         //     recording_url: response.data.recordingUrl,
    //         //     customer_phone: response?.data?.customer?.number,
    //         //     call_status: response.data.status,
    //         //     call_end_reason: response.data.endedReason,
    //         //     call_summary: response.data?.summary,
    //         //     messages: response.data.messages ? response.data.messages.slice(1) : []
    //         // }
    //         // return data;
    //         const phoneCall = await this.prisma.phoneCall.findUnique({
    //             where: {
    //                 id: callId,
    //                 user_id: userId, // Ensure the call belongs to the user
    //             },
    //             include: {
    //                 messages: {
    //                     orderBy: {
    //                         seconds_from_start: 'asc',
    //                     },
    //                 },
    //             },
    //         });
    
    //         if (!phoneCall) {
    //             throw new AppError(404, 'ClientError', 'Call not found or not authorized to access');
    //         }
    
    //         return phoneCall;
            
    //     } catch (err) {
    //         throw new AppError(500, 'ServerError', 'Failed to fetch call details', err);
    //     }
    // }

    // async getUserWithId(userId: number): Promise<z.infer<typeof userDtoSchema> | null> {
    //     try {
    //         let user = await this.userRepository.getUserWithId(userId);
    //         if (user) return this.getUserDto(user);
    //         return null;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async getUsers(limit: number, offset: number, withCount: boolean): Promise<UsersResponse> {
    //     try {
    //         let users = {} as UsersResponse;

    //         if (withCount) {
    //             const response = await this.userRepository.getUsersWithTotalCount(limit, offset);
    //             users = {
    //                 type: 'withCount',
    //                 data: response.data.map((user: z.infer<typeof userSchema>) => this.getUserDto(user)),
    //                 totalCount: response.totalCount
    //             };
    //         } else {
    //             const response = await this.userRepository.getUsers(limit, offset);
    //             users = {
    //                 type: 'withoutCount',
    //                 data: response.map((user) => this.getUserDto(user))
    //             };
    //         }

    //         return users;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong while fetching users', err);
    //     }
    // }

    // async createUser(
    //     userObject: z.infer<typeof userCreateSchema>,
    //     loggedInUserJwtPayload: AppJwtPayload
    // ): Promise<z.infer<typeof userDtoSchema>> {
    //     try {
    //         // if (userObject.role === 'SUPER_ADMIN' && loggedInUserJwtPayload.role !== 'SUPER_ADMIN') {
    //         // if (loggedInUserJwtPayload.role !== 'SUPER_ADMIN') {
    //         //     throw new AppError(403, 'ClientError', 'Only superadmins are allowed to create user');
    //         // }
    //         userObject.password = await bcrypt.hash(userObject.password, 10);
    //         const user = await this.userRepository.createUser(userObject);
    //         return this.getUserDto(user);
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async updateUser(
    //     userId: number,
    //     updatedUserFields: Partial<z.infer<typeof userSchema>>,
    //     loggedInUserJwtPayload: AppJwtPayload
    // ): Promise<z.infer<typeof userDtoSchema>> {
    //     try {
    //         // if (Object.keys(updatedUserFields).includes('password'))
    //         //     throw new AppError(400, 'ClientError', 'Password updates are not allowed from here');

    //         if (updatedUserFields.role === 'SUPER_ADMIN' && loggedInUserJwtPayload.role !== 'SUPER_ADMIN') {
    //             throw new AppError(403, 'ClientError', 'Only superadmins can promote a user to SUPER_ADMIN');
    //         }
    //         if (updatedUserFields.password) {
    //             updatedUserFields.password = await bcrypt.hash(updatedUserFields.password, 10);
    //         }
    //         const user = await this.userRepository.updateUser(userId, updatedUserFields);
    //         return user;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // async deleteUser(userId: 'all' | number) {
    //     try {
    //         await this.userRepository.deleteUser(userId);
    //         return true;
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }

    // getUserDto(user: z.infer<typeof userSchema>): z.infer<typeof userDtoSchema> {
    //     const { password, vapi_key, ...userDto } = user;
    //     return userDto;
    // }

    // async login(email: string, password: string): Promise<{ email: string; role: string; token: string } | null> {
    //     try {
    //         const user = await this.userRepository.getUserWithEmail(email);
    //         if (!user || !user.is_active) return null;
    //         const isPasswordCorrect = await bcrypt.compare(password, user.password);
    //         if (!isPasswordCorrect) return null;

    //         const jwtPayload = {
    //             userId: user.id,
    //             email: user.email,
    //             role: user.role
    //         };
    //         const token = jwt.sign(jwtPayload, process.env.JWT_SECRET!);
    //         return { email: user.email, role: user.role, token };
    //     } catch (err) {
    //         if (err instanceof AppError) throw err;
    //         throw new AppError(500, 'ServerError', 'Something went wrong', err);
    //     }
    // }
}

export default UserService;